//---------------------------------------------------------------------------

#ifndef CFFTH
#define CFFTH
//---------------------------------------------------------------------------
#include <SysUtils.hpp>
#include <Controls.hpp>
#include <Classes.hpp>
#include <Forms.hpp>
#include <complex.h>
#include "fftw.h"

#define pi_const 3.14159265359
#define Complex complex<float>
#ifdef __cplusplus
  #include <complex.h>
#endif

using namespace std;

//---------------------------------------------------------------------------
class PACKAGE TCFFT : public TComponent
{
private:
     fftw_plan pfftplan;
     int FFTSize;
     bool __fastcall fftw_fwd_bckwd(fftw_complex* in, fftw_complex* out);
     bool __fastcall ipFFT_slow(Complex *c1);
     bool __fastcall ipFFT_very_slow(Complex *c1);

public:
     enum TFFTWeighting {
       NoWeighting,
       Hamming,
       Hanning,
       Cheby,
       Taylor
     };
     enum TFFTDirection {
       Forward,
       Backward
     };
     enum TPlanType {
       Estimate,
       Measure
     };
     enum TNormalization {
       None,
       SqrtN,
       N,
       NInv,
       SqrtNInv,
       ConstantAmplitude,
       ConstantPower
     };

private:
   int CurrentWeightingSize;
   TFFTWeighting  CurrentFFTWeighting;
   double *Weighting;
   bool __fastcall CurrentWeightingValid();
   bool __fastcall CreateWeighting();
   float CurrentTaylorN;
   float CurrentSidelobeLevel;
   bool __fastcall TaylorWeights(double *window, double sidelobes, int nbar, int N);
   bool __fastcall ChebyWeights(double *window, double sidelobes, int N);

protected:
   TFFTDirection  FFFTDirection;
   TPlanType      FPlanType;
   TFFTWeighting  FFFTWeighting;
   int            FDataSize;
   int            FTaylorN;
   float          FSidelobeLevel;
   TNormalization FNormalization;
   bool __fastcall Normalize(fftw_complex *out);
   bool           FGPL;
public:
   __fastcall TCFFT(TComponent* Owner);
   __fastcall ~TCFFT();
__published:
     __property TFFTDirection FFTDirection = {read=FFFTDirection, write=FFFTDirection, default=Forward};
     __property TPlanType PlanType = {read=FPlanType, write=FPlanType, default=Estimate};
     __property TFFTWeighting FFTWeighting = {read=FFFTWeighting, write=FFFTWeighting, default=NoWeighting};
     __property int FFTDataSize = {read=FDataSize, write=FDataSize, default=0};
     __property int TaylorN = {read=FTaylorN, write=FTaylorN, default=0};
     __property float SidelobeLevel = {read=FSidelobeLevel, write=FSidelobeLevel, default=0};
     __property TNormalization Normalization = {read=FNormalization, write=FNormalization, default=SqrtN};
     __property bool GPL = {read=FGPL, write=FGPL, default = false};
     bool __fastcall CreatePlan(int N);
     bool __fastcall DestroyPlan();
     bool __fastcall FFT(fftw_complex *in, fftw_complex *out);
     bool __fastcall FFT(complex<double> *in, complex<double> *out);
     bool __fastcall FFT(complex<float> *in, complex<float> *out);

     // In-place versions
     bool __fastcall FFT(fftw_complex *in);
     bool __fastcall FFT(complex<double> *in);
     bool __fastcall FFT(complex<float> *in);

};
//---------------------------------------------------------------------------
#endif
